@@ -1,14 +1,16 @@
 package Net::Joker::DMAPI;
 
-our $VERSION = '0.04';
+our $VERSION = '0.05';
 use strict;
 use 5.010;
+use Carp;
 use Data::Censor;
 use Data::Dump;
 use DateTime;
 use Hash::Merge;
 use LWP::UserAgent;
-use Moose;
+use Moo;
+use MooX::Types::MooseLike::Base qw(:all);
 use URI;
 
 =head1 NAME
@@ -55,7 +57,7 @@ Your Joker account username.
 
 has username => (
     is => 'rw',
-    isa => 'Str',
+    isa => Str,
 );
 
 =item password
@@ -66,7 +68,7 @@ Your Joker account password
 
 has password => (
     is => 'rw',
-    isa => 'Str',
+    isa => Str,
 );
 
 =item debug
@@ -80,7 +82,7 @@ output to STDOUT as well as passed to the C<logger> coderef (if provided).
 
 has debug => (
     is => 'rw',
-    isa => 'Str',
+    isa => Str,
     default => 0,
 );
 
@@ -93,8 +95,8 @@ need to supply this unless you have a specific need to do so.
 
 has ua => (
     is => 'rw',
-    isa => 'LWP::UserAgent',
-    lazy_build => 1,
+    isa => InstanceOf['LWP::UserAgent'],
+    builder => 1,
 );
 sub _build_ua {
     my $ua = LWP::UserAgent->new;
@@ -112,7 +114,7 @@ URL.
 
 has dmapi_url => (
     is => 'rw',
-    isa => 'Str',
+    isa => Str,
     default => 'https://dmapi.joker.com/request',
 );
 
@@ -125,7 +127,7 @@ response from the Joker API is received.
 
 has balance => (
     is => 'rw',
-    isa => 'Str',
+    isa => Str,
 );
 
 =item available_tlds_list
@@ -139,7 +141,7 @@ call C<login> for you first then return the list.
 
 has available_tlds_list => (
     is => 'rw',
-    isa => 'ArrayRef',
+    isa => ArrayRef,
 );
 
 
@@ -156,13 +158,13 @@ C<info>, C<error>), and the message.
 
 has logger => (
     is => 'rw',
-    isa => 'CodeRef',
+    isa => CodeRef,
     predicate => 'has_logger',
 );
 
 has auth_sid => (
     is => 'rw',
-    isa => 'Str',
+    isa => Str,
     default => '',
     predicate => 'has_auth_sid',
 );
@@ -237,6 +239,7 @@ you'll then need to use to poll for the result.
 sub do_request {
     my ($self, $method, $params) = @_;
 
+    $params ||= {};
     my $url = $self->_form_request_url($method, $params);
     $self->_log(
         info => "Calling $method with params: "
@@ -294,6 +297,22 @@ sub available_tlds {
     return $self->available_tlds_list;
 }
 
+=item account_balance
+
+Returns the current balance of your Joker account.  The C<balance> attribute is
+automatically updated after every API call; this method is simply provided to
+ensure you're logged in and return the balance - useful if you want to monitor
+the balance from a Nagios plugin, say, rather than just seeing what the balance
+was after making another API call.
+
+=cut
+
+sub account_balance {
+    my $self = shift;
+    $self->login;
+    return $self->balance;
+}
+
 =item query_whois
 
 A convenient method to call the DMAPI C<query_whois> method, and return the
@@ -313,7 +332,14 @@ Datetimes returned by Joker are automatically inflated to DateTime objects.
 
 sub query_whois {
     my ($self, $params) = @_;
-
+    my @acceptable_params = qw(domain contact host);
+    my @specs = grep { defined $_ } map { $params->{$_} } @acceptable_params;
+    if (scalar @specs != 1) {
+        Carp::croak(
+            "query_whois must be called with exactly one of the params: "
+            . join ',', @acceptable_params
+        );
+    }
     $self->login;
     my $result = $self->do_request('query-whois', $params);
 
@@ -337,7 +363,7 @@ sub expiry_date {
 sub _form_request_url {
     my ($self, $method, $args) = @_;
     my $uri = URI->new($self->dmapi_url . "/$method");
-    $uri->query_form({ 'auth-sid' => $self->auth_sid, %$args });
+    $uri->query_form({ 'auth-sid' => $self->auth_sid, %{ $args || {} } });
     return $uri->canonical;
 }
 
